@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -41,6 +41,7 @@
 #include "prefs.h"
 #include "account.h"
 #include "prefs_account.h"
+#include "sylmain.h"
 
 static GList *folder_list = NULL;
 
@@ -109,6 +110,7 @@ void folder_remote_folder_init(Folder *folder, const gchar *name,
 {
 	folder_init(folder, name);
 	REMOTE_FOLDER(folder)->session = NULL;
+	REMOTE_FOLDER(folder)->remove_cache_on_destroy = TRUE;
 }
 
 void folder_destroy(Folder *folder)
@@ -472,6 +474,9 @@ void folder_write_list(void)
 
 	if (prefs_file_close(pfile) < 0)
 		g_warning("failed to write folder list.\n");
+
+	if (syl_app_get())
+		g_signal_emit_by_name(syl_app_get(), "folderlist-updated");
 }
 
 struct TotalMsgStatus
@@ -644,10 +649,19 @@ FolderItem *folder_find_child_item_by_name(FolderItem *item, const gchar *name)
 {
 	GNode *node;
 	FolderItem *child;
+	const gchar *base;
+
+	if (!name)
+		return NULL;
 
 	for (node = item->node->children; node != NULL; node = node->next) {
 		child = FOLDER_ITEM(node->data);
-		if (strcmp2(g_basename(child->path), name) == 0)
+		base = g_basename(child->path);
+#ifdef G_OS_WIN32
+		if (base && g_ascii_strcasecmp(base, name) == 0)
+#else
+		if (strcmp2(base, name) == 0)
+#endif
 			return child;
 	}
 
@@ -1140,6 +1154,37 @@ gint folder_item_add_msgs(FolderItem *dest, GSList *file_list,
 				       first);
 }
 
+gint folder_item_add_msg_msginfo(FolderItem *dest, MsgInfo *msginfo,
+				 gboolean remove_source)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msginfo != NULL, -1);
+	g_return_val_if_fail(msginfo->file_path != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->add_msg_msginfo != NULL, -1);
+
+	folder = dest->folder;
+
+	return folder->klass->add_msg_msginfo(folder, dest, msginfo,
+					      remove_source);
+}
+
+gint folder_item_add_msgs_msginfo(FolderItem *dest, GSList *msglist,
+				  gboolean remove_source, gint *first)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msglist != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->add_msgs_msginfo != NULL, -1);
+
+	folder = dest->folder;
+
+	return folder->klass->add_msgs_msginfo(folder, dest, msglist,
+					       remove_source, first);
+}
+
 gint folder_item_move_msg(FolderItem *dest, MsgInfo *msginfo)
 {
 	Folder *folder;
@@ -1423,6 +1468,10 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 				qsearch_cond_type = 5;
 			else if (!strcmp(attr->value, "last5"))
 				qsearch_cond_type = 6;
+			else if (!strcmp(attr->value, "last7"))
+				qsearch_cond_type = 7;
+			else if (!strcmp(attr->value, "in-addressbook"))
+				qsearch_cond_type = 8;
 		} else if (!strcmp(attr->name, "account_id")) {
 			account = account_find_from_id(atoi(attr->value));
 			if (!account) g_warning("account_id: %s not found\n",
@@ -1600,7 +1649,8 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 					"from", "subject", "score", "label",
 					"mark", "unread", "mime", "to"};
 	static gchar *qsearch_cond_str[] = {"all", "unread", "mark", "clabel",
-					    "mime", "w1day", "last5"};
+					    "mime", "w1day", "last5", "last7",
+					    "in-addressbook"};
 
 	g_return_if_fail(node != NULL);
 	g_return_if_fail(fp != NULL);
@@ -1659,7 +1709,7 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 				fprintf(fp, " sort_type=\"descending\"");
 		}
 		if (item->qsearch_cond_type > 0 &&
-		    item->qsearch_cond_type < 7) {
+		    item->qsearch_cond_type < 9) {
 			fprintf(fp, " qsearch_cond=\"%s\"",
 				qsearch_cond_str[item->qsearch_cond_type]);
 		}
