@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,53 +35,55 @@
 
 #include "pop.h"
 #include "md5.h"
+#include "prefs.h"
 #include "prefs_account.h"
 #include "utils.h"
 #include "recv.h"
 
-static gint pop3_greeting_recv		(Pop3Session *session,
-					 const gchar *msg);
-static gint pop3_getauth_user_send	(Pop3Session *session);
-static gint pop3_getauth_pass_send	(Pop3Session *session);
-static gint pop3_getauth_apop_send	(Pop3Session *session);
+gint pop3_greeting_recv		(Pop3Session *session,
+				 const gchar *msg);
+gint pop3_getauth_user_send	(Pop3Session *session);
+gint pop3_getauth_pass_send	(Pop3Session *session);
+gint pop3_getauth_apop_send	(Pop3Session *session);
 #if USE_SSL
-static gint pop3_stls_send		(Pop3Session *session);
-static gint pop3_stls_recv		(Pop3Session *session);
+gint pop3_stls_send		(Pop3Session *session);
+gint pop3_stls_recv		(Pop3Session *session);
 #endif
-static gint pop3_getrange_stat_send	(Pop3Session *session);
-static gint pop3_getrange_stat_recv	(Pop3Session *session,
-					 const gchar *msg);
-static gint pop3_getrange_last_send	(Pop3Session *session);
-static gint pop3_getrange_last_recv	(Pop3Session *session,
-					 const gchar *msg);
-static gint pop3_getrange_uidl_send	(Pop3Session *session);
-static gint pop3_getrange_uidl_recv	(Pop3Session *session,
-					 const gchar *data,
-					 guint        len);
-static gint pop3_getsize_list_send	(Pop3Session *session);
-static gint pop3_getsize_list_recv	(Pop3Session *session,
-					 const gchar *data,
-					 guint        len);
-static gint pop3_retr_send		(Pop3Session *session);
-static gint pop3_retr_recv		(Pop3Session *session,
-					 FILE	     *fp,
-					 guint        len);
-static gint pop3_delete_send		(Pop3Session *session);
-static gint pop3_delete_recv		(Pop3Session *session);
-static gint pop3_logout_send		(Pop3Session *session);
+gint pop3_getrange_stat_send	(Pop3Session *session);
+gint pop3_getrange_stat_recv	(Pop3Session *session,
+				 const gchar *msg);
+gint pop3_getrange_last_send	(Pop3Session *session);
+gint pop3_getrange_last_recv	(Pop3Session *session,
+				 const gchar *msg);
+gint pop3_getrange_uidl_send	(Pop3Session *session);
+gint pop3_getrange_uidl_recv	(Pop3Session *session,
+				 const gchar *data,
+				 guint        len);
+gint pop3_getsize_list_send	(Pop3Session *session);
+gint pop3_getsize_list_recv	(Pop3Session *session,
+				 const gchar *data,
+				 guint        len);
+gint pop3_retr_send		(Pop3Session *session);
+gint pop3_retr_recv		(Pop3Session *session,
+				 FILE	     *fp,
+				 guint        len);
+gint pop3_delete_send		(Pop3Session *session);
+gint pop3_delete_recv		(Pop3Session *session);
+gint pop3_logout_send		(Pop3Session *session);
 
-static void pop3_gen_send		(Pop3Session	*session,
-					 const gchar	*format, ...);
+void pop3_gen_send		(Pop3Session	*session,
+				 const gchar	*format, ...);
 
 static void pop3_session_destroy	(Session	*session);
 
-static gint pop3_write_msg_to_file	(const gchar	*file,
-					 FILE		*src_fp,
-					 guint		 len);
+gint pop3_write_msg_to_file	(const gchar	*file,
+				 FILE		*src_fp,
+				 guint		 len);
 
 static Pop3State pop3_lookup_next	(Pop3Session	*session);
-static Pop3ErrorValue pop3_ok		(Pop3Session	*session,
-					 const gchar	*msg);
+
+Pop3ErrorValue pop3_ok		(Pop3Session	*session,
+				 const gchar	*msg);
 
 static gint pop3_session_recv_msg		(Session	*session,
 						 const gchar	*msg);
@@ -94,7 +96,7 @@ static gint pop3_session_recv_data_as_file_finished
 						 guint		 len);
 
 
-static gint pop3_greeting_recv(Pop3Session *session, const gchar *msg)
+gint pop3_greeting_recv(Pop3Session *session, const gchar *msg)
 {
 	session->state = POP3_GREETING;
 
@@ -103,14 +105,14 @@ static gint pop3_greeting_recv(Pop3Session *session, const gchar *msg)
 }
 
 #if USE_SSL
-static gint pop3_stls_send(Pop3Session *session)
+gint pop3_stls_send(Pop3Session *session)
 {
 	session->state = POP3_STLS;
 	pop3_gen_send(session, "STLS");
 	return PS_SUCCESS;
 }
 
-static gint pop3_stls_recv(Pop3Session *session)
+gint pop3_stls_recv(Pop3Session *session)
 {
 	if (session_start_tls(SESSION(session)) < 0) {
 		session->error_val = PS_SOCKET;
@@ -120,7 +122,7 @@ static gint pop3_stls_recv(Pop3Session *session)
 }
 #endif /* USE_SSL */
 
-static gint pop3_getauth_user_send(Pop3Session *session)
+gint pop3_getauth_user_send(Pop3Session *session)
 {
 	g_return_val_if_fail(session->user != NULL, -1);
 
@@ -129,7 +131,7 @@ static gint pop3_getauth_user_send(Pop3Session *session)
 	return PS_SUCCESS;
 }
 
-static gint pop3_getauth_pass_send(Pop3Session *session)
+gint pop3_getauth_pass_send(Pop3Session *session)
 {
 	g_return_val_if_fail(session->pass != NULL, -1);
 
@@ -138,7 +140,7 @@ static gint pop3_getauth_pass_send(Pop3Session *session)
 	return PS_SUCCESS;
 }
 
-static gint pop3_getauth_apop_send(Pop3Session *session)
+gint pop3_getauth_apop_send(Pop3Session *session)
 {
 	gchar *start, *end;
 	gchar *apop_str;
@@ -165,6 +167,12 @@ static gint pop3_getauth_apop_send(Pop3Session *session)
 
 	*(end + 1) = '\0';
 
+	if (!is_ascii_str(start) || strchr(start, '@') == NULL) {
+		log_warning(_("Invalid timestamp in greeting\n"));
+		session->error_val = PS_PROTOCOL;
+		return -1;
+	}
+
 	apop_str = g_strconcat(start, session->pass, NULL);
 	md5 = s_gnet_md5_new((guchar *)apop_str, strlen(apop_str));
 	md5sum = s_gnet_md5_get_string(md5);
@@ -178,14 +186,14 @@ static gint pop3_getauth_apop_send(Pop3Session *session)
 	return PS_SUCCESS;
 }
 
-static gint pop3_getrange_stat_send(Pop3Session *session)
+gint pop3_getrange_stat_send(Pop3Session *session)
 {
 	session->state = POP3_GETRANGE_STAT;
 	pop3_gen_send(session, "STAT");
 	return PS_SUCCESS;
 }
 
-static gint pop3_getrange_stat_recv(Pop3Session *session, const gchar *msg)
+gint pop3_getrange_stat_recv(Pop3Session *session, const gchar *msg)
 {
 	if (sscanf(msg, "%d %lld", &session->count, &session->total_bytes) != 2) {
 		log_warning(_("POP3 protocol error\n"));
@@ -203,14 +211,14 @@ static gint pop3_getrange_stat_recv(Pop3Session *session, const gchar *msg)
 	return PS_SUCCESS;
 }
 
-static gint pop3_getrange_last_send(Pop3Session *session)
+gint pop3_getrange_last_send(Pop3Session *session)
 {
 	session->state = POP3_GETRANGE_LAST;
 	pop3_gen_send(session, "LAST");
 	return PS_SUCCESS;
 }
 
-static gint pop3_getrange_last_recv(Pop3Session *session, const gchar *msg)
+gint pop3_getrange_last_recv(Pop3Session *session, const gchar *msg)
 {
 	gint last;
 
@@ -229,15 +237,14 @@ static gint pop3_getrange_last_recv(Pop3Session *session, const gchar *msg)
 	return PS_SUCCESS;
 }
 
-static gint pop3_getrange_uidl_send(Pop3Session *session)
+gint pop3_getrange_uidl_send(Pop3Session *session)
 {
 	session->state = POP3_GETRANGE_UIDL;
 	pop3_gen_send(session, "UIDL");
 	return PS_SUCCESS;
 }
 
-static gint pop3_getrange_uidl_recv(Pop3Session *session, const gchar *data,
-				    guint len)
+gint pop3_getrange_uidl_recv(Pop3Session *session, const gchar *data, guint len)
 {
 	gchar id[IDLEN + 1];
 	gchar buf[POPBUFSIZE];
@@ -284,15 +291,14 @@ static gint pop3_getrange_uidl_recv(Pop3Session *session, const gchar *data,
 	return PS_SUCCESS;
 }
 
-static gint pop3_getsize_list_send(Pop3Session *session)
+gint pop3_getsize_list_send(Pop3Session *session)
 {
 	session->state = POP3_GETSIZE_LIST;
 	pop3_gen_send(session, "LIST");
 	return PS_SUCCESS;
 }
 
-static gint pop3_getsize_list_recv(Pop3Session *session, const gchar *data,
-				   guint len)
+gint pop3_getsize_list_recv(Pop3Session *session, const gchar *data, guint len)
 {
 	gchar buf[POPBUFSIZE];
 	gint buf_len;
@@ -325,14 +331,14 @@ static gint pop3_getsize_list_recv(Pop3Session *session, const gchar *data,
 	return PS_SUCCESS;
 }
 
-static gint pop3_retr_send(Pop3Session *session)
+gint pop3_retr_send(Pop3Session *session)
 {
 	session->state = POP3_RETR;
 	pop3_gen_send(session, "RETR %d", session->cur_msg);
 	return PS_SUCCESS;
 }
 
-static gint pop3_retr_recv(Pop3Session *session, FILE *fp, guint len)
+gint pop3_retr_recv(Pop3Session *session, FILE *fp, guint len)
 {
 	gchar *file;
 	gint drop_ok;
@@ -365,28 +371,28 @@ static gint pop3_retr_recv(Pop3Session *session, FILE *fp, guint len)
 	return PS_SUCCESS;
 }
 
-static gint pop3_delete_send(Pop3Session *session)
+gint pop3_delete_send(Pop3Session *session)
 {
 	session->state = POP3_DELETE;
 	pop3_gen_send(session, "DELE %d", session->cur_msg);
 	return PS_SUCCESS;
 }
 
-static gint pop3_delete_recv(Pop3Session *session)
+gint pop3_delete_recv(Pop3Session *session)
 {
 	session->msg[session->cur_msg].recv_time = RECV_TIME_DELETE;
 	session->msg[session->cur_msg].deleted = TRUE;
 	return PS_SUCCESS;
 }
 
-static gint pop3_logout_send(Pop3Session *session)
+gint pop3_logout_send(Pop3Session *session)
 {
 	session->state = POP3_LOGOUT;
 	pop3_gen_send(session, "QUIT");
 	return PS_SUCCESS;
 }
 
-static void pop3_gen_send(Pop3Session *session, const gchar *format, ...)
+void pop3_gen_send(Pop3Session *session, const gchar *format, ...)
 {
 	gchar buf[POPBUFSIZE + 1];
 	va_list args;
@@ -430,6 +436,22 @@ Session *pop3_session_new(PrefsAccount *account)
 	session->error_val = PS_SUCCESS;
 	session->error_msg = NULL;
 
+	session->user = g_strdup(account->userid);
+	session->pass = account->passwd ? g_strdup(account->passwd) :
+		account->tmp_pass ? g_strdup(account->tmp_pass) : NULL;
+
+	SESSION(session)->server = g_strdup(account->recv_server);
+
+#if USE_SSL
+	SESSION(session)->port = account->set_popport ?
+		account->popport : account->ssl_pop == SSL_TUNNEL ? 995 : 110;
+	SESSION(session)->ssl_type = account->ssl_pop;
+	if (account->ssl_pop != SSL_NONE)
+		SESSION(session)->nonblocking = account->use_nonblocking_ssl;
+#else
+        SESSION(session)->port = account->set_popport ? account->popport : 110;
+#endif
+
 	return SESSION(session);
 }
 
@@ -504,7 +526,7 @@ GHashTable *pop3_get_uidl_table(PrefsAccount *ac_prefs)
 gint pop3_write_uidl_list(Pop3Session *session)
 {
 	gchar *path;
-	FILE *fp;
+	PrefFile *pfile;
 	Pop3MsgInfo *msg;
 	gint n;
 	gchar *uid;
@@ -517,11 +539,11 @@ gint pop3_write_uidl_list(Pop3Session *session)
 			   session->ac_prefs->recv_server,
 			   "-", uid, NULL);
 	g_free(uid);
-	if ((fp = g_fopen(path, "wb")) == NULL) {
-		FILE_OP_ERROR(path, "fopen");
+	if ((pfile = prefs_file_open(path)) == NULL) {
 		g_free(path);
 		return -1;
 	}
+	prefs_file_set_backup_generation(pfile, 0);
 
 	for (n = 1; n <= session->count; n++) {
 		msg = &session->msg[n];
@@ -529,16 +551,18 @@ gint pop3_write_uidl_list(Pop3Session *session)
 			continue;
 		if (session->state == POP3_DONE && msg->deleted)
 			continue;
-		fprintf(fp, "%s\t%ld\n", msg->uidl, msg->recv_time);
+		fprintf(pfile->fp, "%s\t%ld\n", msg->uidl, msg->recv_time);
 	}
 
-	if (fclose(fp) == EOF) FILE_OP_ERROR(path, "fclose");
+	if (prefs_file_close(pfile) < 0)
+		g_warning("%s: failed to write UIDL list.\n", path);
+
 	g_free(path);
 
 	return 0;
 }
 
-static gint pop3_write_msg_to_file(const gchar *file, FILE *src_fp, guint len)
+gint pop3_write_msg_to_file(const gchar *file, FILE *src_fp, guint len)
 {
 	FILE *fp;
 	gchar buf[BUFFSIZE];
@@ -627,10 +651,12 @@ static Pop3State pop3_lookup_next(Pop3Session *session)
 			return POP3_DELETE;
 		}
 
-		if (size_limit_over)
+		if (size_limit_over && !msg->received) {
 			log_print
 				(_("POP3: Skipping message %d (%d bytes)\n"),
 				  session->cur_msg, size);
+			session->skipped_num++;
+		}
 
 		if (size == 0 || msg->received || size_limit_over) {
 			session->cur_total_bytes += size;
@@ -644,10 +670,11 @@ static Pop3State pop3_lookup_next(Pop3Session *session)
 	}
 
 	pop3_retr_send(session);
+
 	return POP3_RETR;
 }
 
-static Pop3ErrorValue pop3_ok(Pop3Session *session, const gchar *msg)
+Pop3ErrorValue pop3_ok(Pop3Session *session, const gchar *msg)
 {
 	Pop3ErrorValue ok;
 
@@ -703,7 +730,7 @@ static Pop3ErrorValue pop3_ok(Pop3Session *session, const gchar *msg)
 static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 {
 	Pop3Session *pop3_session = POP3_SESSION(session);
-	Pop3ErrorValue val = PS_SUCCESS;
+	gint val = PS_SUCCESS;
 	const gchar *body;
 
 	body = msg;
@@ -728,73 +755,77 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 	switch (pop3_session->state) {
 	case POP3_READY:
 	case POP3_GREETING:
-		pop3_greeting_recv(pop3_session, body);
+		val = pop3_greeting_recv(pop3_session, body);
 #if USE_SSL
 		if (pop3_session->ac_prefs->ssl_pop == SSL_STARTTLS)
-			pop3_stls_send(pop3_session);
+			val = pop3_stls_send(pop3_session);
 		else
 #endif
 		if (pop3_session->ac_prefs->use_apop_auth)
-			pop3_getauth_apop_send(pop3_session);
+			val = pop3_getauth_apop_send(pop3_session);
 		else
-			pop3_getauth_user_send(pop3_session);
+			val = pop3_getauth_user_send(pop3_session);
 		break;
 #if USE_SSL
 	case POP3_STLS:
-		if (pop3_stls_recv(pop3_session) != PS_SUCCESS)
+		if ((val = pop3_stls_recv(pop3_session)) != PS_SUCCESS)
 			return -1;
 		if (pop3_session->ac_prefs->use_apop_auth)
-			pop3_getauth_apop_send(pop3_session);
+			val = pop3_getauth_apop_send(pop3_session);
 		else
-			pop3_getauth_user_send(pop3_session);
+			val = pop3_getauth_user_send(pop3_session);
 		break;
 #endif
 	case POP3_GETAUTH_USER:
-		pop3_getauth_pass_send(pop3_session);
+		val = pop3_getauth_pass_send(pop3_session);
 		break;
 	case POP3_GETAUTH_PASS:
 	case POP3_GETAUTH_APOP:
-		pop3_getrange_stat_send(pop3_session);
+		if (pop3_session->auth_only)
+			val = pop3_logout_send(pop3_session);
+		else
+			val = pop3_getrange_stat_send(pop3_session);
 		break;
 	case POP3_GETRANGE_STAT:
-		if (pop3_getrange_stat_recv(pop3_session, body) < 0)
+		if ((val = pop3_getrange_stat_recv(pop3_session, body)) < 0)
 			return -1;
 		if (pop3_session->count > 0)
-			pop3_getrange_uidl_send(pop3_session);
+			val = pop3_getrange_uidl_send(pop3_session);
 		else
-			pop3_logout_send(pop3_session);
+			val = pop3_logout_send(pop3_session);
 		break;
 	case POP3_GETRANGE_LAST:
 		if (val == PS_NOTSUPPORTED)
 			pop3_session->error_val = PS_SUCCESS;
-		else if (pop3_getrange_last_recv(pop3_session, body) < 0)
+		else if ((val = pop3_getrange_last_recv
+				(pop3_session, body)) < 0)
 			return -1;
 		if (pop3_session->cur_msg > 0)
-			pop3_getsize_list_send(pop3_session);
+			val = pop3_getsize_list_send(pop3_session);
 		else
-			pop3_logout_send(pop3_session);
+			val = pop3_logout_send(pop3_session);
 		break;
 	case POP3_GETRANGE_UIDL:
 		if (val == PS_NOTSUPPORTED) {
 			pop3_session->error_val = PS_SUCCESS;
-			pop3_getrange_last_send(pop3_session);
+			val = pop3_getrange_last_send(pop3_session);
 		} else {
 			pop3_session->state = POP3_GETRANGE_UIDL_RECV;
-			session_recv_data(session, 0, ".\r\n");
+			val = session_recv_data(session, 0, ".\r\n");
 		}
 		break;
 	case POP3_GETSIZE_LIST:
 		pop3_session->state = POP3_GETSIZE_LIST_RECV;
-		session_recv_data(session, 0, ".\r\n");
+		val = session_recv_data(session, 0, ".\r\n");
 		break;
 	case POP3_RETR:
 		pop3_session->state = POP3_RETR_RECV;
-		session_recv_data_as_file(session, 0, ".\r\n");
+		val = session_recv_data_as_file(session, 0, ".\r\n");
 		break;
 	case POP3_DELETE:
-		pop3_delete_recv(pop3_session);
+		val = pop3_delete_recv(pop3_session);
 		if (pop3_session->cur_msg == pop3_session->count)
-			pop3_logout_send(pop3_session);
+			val = pop3_logout_send(pop3_session);
 		else {
 			pop3_session->cur_msg++;
 			if (pop3_lookup_next(pop3_session) == POP3_ERROR)
@@ -810,7 +841,10 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 		return -1;
 	}
 
-	return 0;
+	if (val == PS_SUCCESS)
+		return 0;
+	else
+		return -1;
 }
 
 static gint pop3_session_recv_data_finished(Session *session, guchar *data,
