@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2008 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2010 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -289,6 +289,11 @@ struct _PrefsCommon
 	gboolean use_extsend;
 	gchar *extsend_cmd;
 
+	/* Update check */
+	gboolean auto_update_check;
+	gboolean use_http_proxy;
+	gchar *http_proxy_host;
+
 	/* Advanced */
 	gboolean strict_cache_check;
 	gint io_timeout_secs;
@@ -303,6 +308,12 @@ struct _PrefsCommon
 
 	/* Online / Offline */
 	gboolean online_mode;
+
+	/* Append new members here */
+	gboolean folder_col_visible[4];      /* Display */
+	gboolean reply_address_only;         /* Compose */
+	gboolean recipients_autoreg;         /* Send */
+	gboolean enable_address_completion;  /* Compose */
 };
 
 extern PrefsCommon prefs_common;
