@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -119,10 +119,20 @@ void session_init(Session *session)
 
 gint session_connect(Session *session, const gchar *server, gushort port)
 {
-#ifdef G_OS_UNIX
-	session->server = g_strdup(server);
+#ifndef G_OS_UNIX
+	SockInfo *sock = NULL;
+#endif
+	g_return_val_if_fail(session != NULL, -1);
+	g_return_val_if_fail(server != NULL, -1);
+	g_return_val_if_fail(port > 0, -1);
+
+	if (session->server != server) {
+		g_free(session->server);
+		session->server = g_strdup(server);
+	}
 	session->port = port;
 
+#ifdef G_OS_UNIX
 	session->conn_id = sock_connect_async(server, port, session_connect_cb,
 					      session);
 	if (session->conn_id < 0) {
@@ -132,12 +142,21 @@ gint session_connect(Session *session, const gchar *server, gushort port)
 	}
 
 	return 0;
-#else
-	SockInfo *sock;
-
-	session->server = g_strdup(server);
-	session->port = port;
+#elif USE_THREADS
+	session->conn_id = sock_connect_async_thread(server, port);
+	if (session->conn_id < 0) {
+		g_warning("can't connect to server.");
+		session->state = SESSION_ERROR;
+		return -1;
+	}
+	if (sock_connect_async_thread_wait(session->conn_id, &sock) < 0) {
+		g_warning("can't connect to server.");
+		session->state = SESSION_ERROR;
+		return -1;
+	}
 
+	return session_connect_cb(sock, session);
+#else /* !USE_THREADS */
 	sock = sock_connect(server, port);
 	if (sock == NULL) {
 		g_warning("can't connect to server.");
@@ -284,7 +303,7 @@ static gboolean session_ping_cb(gpointer data)
 
 			debug_print("state machine freeze for 1 second detected, forcing dispatch.\n");
 			save_flags = sock->flags;
-			SOCK_UNSET_FLAGS(sock->flags, SOCK_CHECK_IO);
+			SOCK_UNSET_FLAGS(sock->flags, SYL_SOCK_CHECK_IO);
 			sock->callback(sock, sock->condition, sock->data);
 			sock->flags = save_flags;
 		}
