@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2008 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -81,11 +81,18 @@ struct _SockConnectData {
 	gint id;
 	gchar *hostname;
 	gushort port;
+#ifdef G_OS_UNIX
 	GList *addr_list;
 	GList *cur_addr;
 	SockLookupData *lookup_data;
 	GIOChannel *channel;
 	guint io_tag;
+#endif /* G_OS_UNIX */
+#if USE_THREADS
+	gint flag;
+	GThread *thread;
+	SockInfo *sock;
+#endif /* G_OS_UNIX */
 	SockConnectFunc func;
 	gpointer data;
 };
@@ -617,7 +624,7 @@ static gint fd_check_io(gint fd, GIOCondition cond)
 	}
 }
 
-#ifdef G_OS_UNIX
+#if defined(G_OS_UNIX) && !defined(USE_THREADS)
 static sigjmp_buf jmpenv;
 
 static void timeout_handler(gint sig)
@@ -632,25 +639,61 @@ static gint sock_connect_with_timeout(gint sock,
 				      guint timeout_secs)
 {
 	gint ret;
-#ifdef G_OS_UNIX
-	void (*prev_handler)(gint);
 
-	alarm(0);
-	prev_handler = signal(SIGALRM, timeout_handler);
-	if (sigsetjmp(jmpenv, 1)) {
-		alarm(0);
-		signal(SIGALRM, prev_handler);
-		errno = ETIMEDOUT;
-		return -1;
-	}
-	alarm(timeout_secs);
+#ifdef G_OS_UNIX
+	set_nonblocking_mode(sock, TRUE);
 #endif
 
 	ret = connect(sock, serv_addr, addrlen);
 
 #ifdef G_OS_UNIX
-	alarm(0);
-	signal(SIGALRM, prev_handler);
+	if (ret < 0) {
+		if (EINPROGRESS == errno) {
+			fd_set fds;
+			struct timeval tv;
+
+			tv.tv_sec = timeout_secs;
+			tv.tv_usec = 0;
+			FD_ZERO(&fds);
+			FD_SET(sock, &fds);
+			do {
+				ret = select(sock + 1, NULL, &fds, NULL, &tv);
+			} while (ret < 0 && EINTR == errno);
+			if (ret < 0) {
+				perror("sock_connect_with_timeout: select");
+				return -1;
+			} else if (ret == 0) {
+				debug_print("sock_connect_with_timeout: timeout\n");
+				errno = ETIMEDOUT;
+				return -1;
+			} else {
+				gint val;
+				guint len;
+
+				if (FD_ISSET(sock, &fds)) {
+					ret = 0;
+				} else {
+					debug_print("sock_connect_with_timeout: fd not set\n");
+					return -1;
+				}
+
+				len = sizeof(val);
+				if (getsockopt(sock, SOL_SOCKET, SO_ERROR, &val, &len) < 0) {
+					perror("sock_connect_with_timeout: getsockopt");
+					return -1;
+				}
+				if (val != 0) {
+					debug_print("sock_connect_with_timeout: getsockopt(SOL_SOCKET, SO_ERROR) returned error: %s\n", g_strerror(val));
+					return -1;
+				}
+			}
+		} else {
+			perror("sock_connect_with_timeout: connect");
+			return -1;
+		}
+	}
+
+	set_nonblocking_mode(sock, FALSE);
 #endif
 
 	return ret;
@@ -674,7 +717,7 @@ static void resolver_init(void)
 struct hostent *my_gethostbyname(const gchar *hostname)
 {
 	struct hostent *hp;
-#ifdef G_OS_UNIX
+#if defined(G_OS_UNIX) && !defined(USE_THREADS)
 	void (*prev_handler)(gint);
 
 	alarm(0);
@@ -690,7 +733,7 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 #endif
 
 	if ((hp = gethostbyname(hostname)) == NULL) {
-#ifdef G_OS_UNIX
+#if defined(G_OS_UNIX) && !defined(USE_THREADS)
 		alarm(0);
 		signal(SIGALRM, prev_handler);
 #endif
@@ -699,7 +742,7 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 		return NULL;
 	}
 
-#ifdef G_OS_UNIX
+#if defined(G_OS_UNIX) && !defined(USE_THREADS)
 	alarm(0);
 	signal(SIGALRM, prev_handler);
 #endif
@@ -1319,6 +1362,89 @@ static gint sock_get_address_info_async_cancel(SockLookupData *lookup_data)
 }
 #endif /* G_OS_UNIX */
 
+#if USE_THREADS
+static gpointer sock_connect_async_func(gpointer data)
+{
+	SockConnectData *conn_data = (SockConnectData *)data;
+	gint ret;
+
+	conn_data->sock = sock_connect(conn_data->hostname, conn_data->port);
+
+	if (conn_data->sock) {
+		debug_print("sock_connect_async_func: connected\n");
+		ret = 0;
+	} else {
+		debug_print("sock_connect_async_func: connection failed\n");
+		ret = -1;
+	}
+
+	g_atomic_int_set(&conn_data->flag, 1);
+	g_main_context_wakeup(NULL);
+
+	debug_print("sock_connect_async_func: exit\n");
+	return GINT_TO_POINTER(ret);
+}
+
+gint sock_connect_async_thread(const gchar *hostname, gushort port)
+{
+	static gint id = 1;
+	SockConnectData *data;
+
+	data = g_new0(SockConnectData, 1);
+	data->id = id++;
+	data->hostname = g_strdup(hostname);
+	data->port = port;
+	data->flag = 0;
+	data->sock = NULL;
+
+	data->thread = g_thread_create(sock_connect_async_func, data, TRUE,
+				       NULL);
+	if (!data->thread) {
+		g_free(data->hostname);
+		g_free(data);
+		return -1;
+	}
+
+	sock_connect_data_list = g_list_append(sock_connect_data_list, data);
+
+	return data->id;
+}
+
+gint sock_connect_async_thread_wait(gint id, SockInfo **sock)
+{
+	SockConnectData *conn_data = NULL;
+	GList *cur;
+	gint ret;
+
+	for (cur = sock_connect_data_list; cur != NULL; cur = cur->next) {
+		if (((SockConnectData *)cur->data)->id == id) {
+			conn_data = (SockConnectData *)cur->data;
+			break;
+		}
+	}
+
+	if (!conn_data) {
+		g_warning("sock_connect_async_thread_wait: id %d not found.", id);
+		return -1;
+	}
+
+	debug_print("sock_connect_async_thread_wait: waiting thread\n");
+	while (g_atomic_int_get(&conn_data->flag) == 0)
+		event_loop_iterate();
+
+	ret = GPOINTER_TO_INT(g_thread_join(conn_data->thread));
+	debug_print("sock_connect_async_thread_wait: thread exited with status %d\n", ret);
+
+	*sock = conn_data->sock;
+
+	sock_connect_data_list = g_list_remove(sock_connect_data_list,
+					       conn_data);
+	g_free(conn_data->hostname);
+	g_free(conn_data);
+
+	return ret;
+}
+#endif /* USE_THREADS */
 
 gint sock_printf(SockInfo *sock, const gchar *format, ...)
 {
