@@ -51,6 +51,7 @@
 #include "addrindex.h"
 #include "editaddress.h"
 #include "manage_window.h"
+#include "utils.h"
 
 typedef struct {
 	AddressBookFile	*book;
@@ -254,9 +255,9 @@ static void addressadd_create( void ) {
 
 	gtk_widget_show_all(vbox);
 
-	stock_pixmap_gdk( window, STOCK_PIXMAP_BOOK, &bookXpm, &bookXpmMask );
-	stock_pixmap_gdk( window, STOCK_PIXMAP_DIR_OPEN,
-			  &folderXpm, &folderXpmMask );
+	stock_pixmap_gdk(window, STOCK_PIXMAP_BOOK, &bookXpm, &bookXpmMask);
+	stock_pixmap_gdk(window, STOCK_PIXMAP_FOLDER_OPEN,
+			 &folderXpm, &folderXpmMask);
 }
 
 static void addressadd_load_folder( GtkCTreeNode *parentNode, ItemFolder *parentFolder,
@@ -306,7 +307,10 @@ static void addressadd_load_data( AddressIndex *addrIndex ) {
 			nodeDS = interface->listSource;
 			while( nodeDS ) {
 				ds = nodeDS->data;
-				dsName = g_strdup( addrindex_ds_get_name( ds ) );
+				if ( !strcmp( addrindex_ds_get_name( ds ), ADDR_DS_AUTOREG ) )
+					dsName = g_strdup( _("Auto-registered address") );
+				else
+					dsName = g_strdup( addrindex_ds_get_name( ds ) );
 
 				/* Read address book */
 				if( ! addrindex_ds_get_read_flag( ds ) ) {
@@ -361,6 +365,9 @@ gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, const
 	if( remarks )
 		gtk_label_set_text( GTK_LABEL(addressadd_dlg.label_remarks ), remarks );
 
+	if( ! name )
+		name = "";
+
 	gtk_main();
 	gtk_widget_hide( addressadd_dlg.window );
 
@@ -382,6 +389,53 @@ gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, const
 	return retVal;
 }
 
+gboolean addressadd_autoreg(AddressIndex *addrIndex, const gchar *name,
+			    const gchar *address, const gchar *remarks)
+{
+	ItemPerson *person = NULL;
+	AddressInterface *interface = NULL;
+	AddressDataSource *ds = NULL;
+	AddressBookFile *abf = NULL;
+	GList *node_ds;
+	const gchar *ds_name;
+
+	g_return_val_if_fail(address != NULL, FALSE);
+
+	if (!name)
+		name = "";
+
+	interface = addrindex_get_interface(addrIndex, ADDR_IF_BOOK);
+	if (!interface)
+		return FALSE;
+
+	for (node_ds = interface->listSource; node_ds != NULL;
+	     node_ds = node_ds->next) {
+		ds = node_ds->data;
+		ds_name = addrindex_ds_get_name(ds);
+		if (!ds_name)
+			continue;
+		if (strcmp(ds_name, ADDR_DS_AUTOREG) != 0)
+			continue;
+		debug_print("addressadd_autoreg: AddressDataSource: %s found\n", ds_name);
+
+		if (!addrindex_ds_get_read_flag(ds))
+			addrindex_ds_read_data(ds);
+		abf = ds->rawDataSource;
+	}
+
+	if (!abf)
+		return FALSE;
+
+	person = addrbook_add_contact(abf, NULL, name, address, remarks);
+	if (person) {
+		debug_print("addressadd_autoreg: person added: %s <%s>\n",
+			    name, address);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /*
 * End of Source.
 */
