@@ -60,17 +60,18 @@ struct _AddressBook_win
 {
 	GtkWidget *window;
 	GtkWidget *menubar;
-	GtkWidget *ctree;
-	GtkWidget *clist;
+	GtkWidget *treeview;
+	GtkWidget *listview;
 	GtkWidget *entry;
 	GtkWidget *statusbar;
 
-	GtkWidget *del_btn;
-	GtkWidget *reg_btn;
-	GtkWidget *lup_btn;
 	GtkWidget *to_btn;
 	GtkWidget *cc_btn;
 	GtkWidget *bcc_btn;
+	GtkWidget *del_btn;
+	GtkWidget *reg_btn;
+	GtkWidget *lup_btn;
+	GtkWidget *close_btn;
 
 	GtkWidget *tree_popup;
 	GtkWidget *list_popup;
@@ -78,9 +79,9 @@ struct _AddressBook_win
 	GtkItemFactory *list_factory;
 	GtkItemFactory *menu_factory;
 
-	GtkCTreeNode *treeSelected;
-	GtkCTreeNode *opened;
-	GtkCTreeNode *listSelected;
+	GtkTreeRowReference *tree_selected;
+	GtkTreeRowReference *tree_opened;
+	GtkTreeRowReference *list_selected;
 
 	Compose *target_compose;
 	gint status_cid;
@@ -95,10 +96,8 @@ struct _AddressTypeControlItem {
 	gboolean treeExpand;
 	gboolean treeLeaf;
 	gchar *menuCommand;
-	GdkPixmap *iconXpm;
-	GdkBitmap *maskXpm;
-	GdkPixmap *iconXpmOpen;
-	GdkBitmap *maskXpmOpen;
+	GdkPixbuf *icon_pixbuf;
+	GdkPixbuf *icon_open_pixbuf;
 };
 
 typedef struct _AddressObject	AddressObject;
@@ -115,7 +114,7 @@ struct _AdapterInterface {
 	AddressTypeControlItem *atci;
 	gboolean enabled;
 	gboolean haveLibrary;
-	GtkCTreeNode *treeNode;
+	GtkTreeRowReference *tree_row;
 };
 
 typedef struct _AdapterDSource AdapterDSource;
