@@ -310,7 +310,7 @@ static void foldersel_create(void)
 	gtk_container_add(GTK_CONTAINER(scrolledwin), treeview);
 
 	column = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_spacing(column, 2);
+	gtk_tree_view_column_set_spacing(column, 1);
 	renderer = gtk_cell_renderer_pixbuf_new();
 	g_object_set(renderer, "ypad", 0, NULL);
 	gtk_tree_view_column_pack_start(column, renderer, FALSE);
@@ -344,11 +344,17 @@ static void foldersel_create(void)
 	gtkut_stock_button_set_create(&confirm_area,
 				      &ok_button,     GTK_STOCK_OK,
 				      &cancel_button, GTK_STOCK_CANCEL,
-				      &new_button,    GTK_STOCK_NEW);
+				      NULL, NULL);
 
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_button);
 
+	new_button = gtk_button_new_from_stock(GTK_STOCK_NEW);
+	gtk_widget_show(new_button);
+	gtk_box_pack_start(GTK_BOX(confirm_area), new_button, FALSE, FALSE, 0);
+	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(confirm_area),
+					   new_button, TRUE);
+
 	g_signal_connect(G_OBJECT(ok_button), "clicked",
 			 G_CALLBACK(foldersel_ok), NULL);
 	g_signal_connect(G_OBJECT(cancel_button), "clicked",
@@ -361,11 +367,11 @@ static void foldersel_create(void)
 
 static void foldersel_init(void)
 {
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_CLOSE,
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_CLOSE,
 			 &folder_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_OPEN,
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_OPEN,
 			 &folderopen_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_NOSELECT,
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_NOSELECT,
 			 &foldernoselect_pixbuf);
 }
 
