@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,11 +32,13 @@
 #include <gtk/gtkeditable.h>
 #include <gtk/gtkctree.h>
 #include <gtk/gtktreemodel.h>
+#include <gtk/gtktreestore.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkcombo.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtktextview.h>
 #include <gtk/gtkitemfactory.h>
+#include <gtk/gtkentry.h>
 #include <gtk/gtkstock.h>
 
 #include <stdlib.h>
@@ -81,6 +83,10 @@ struct _ComboButton
 #define GTKUT_CTREE_REFRESH(clist) \
 	GTK_CLIST_GET_CLASS(clist)->refresh(clist)
 
+gboolean gtkut_get_str_size		(GtkWidget	*widget,
+					 const gchar	*str,
+					 gint		*width,
+					 gint		*height);
 gboolean gtkut_get_font_size		(GtkWidget	*widget,
 					 gint		*width,
 					 gint		*height);
@@ -88,6 +94,8 @@ PangoFontDescription *gtkut_get_default_font_desc
 					(void);
 void gtkut_widget_set_small_font_size	(GtkWidget	*widget);
 
+gboolean gtkut_font_can_load		(const gchar	*str);
+
 void gtkut_convert_int_to_gdk_color	(gint		 rgbvalue,
 					 GdkColor	*color);
 
@@ -117,6 +125,7 @@ void gtkut_ctree_set_focus_row		(GtkCTree	*ctree,
 					 GtkCTreeNode	*node);
 void gtkut_clist_set_focus_row		(GtkCList	*clist,
 					 gint		 row);
+void gtkut_clist_set_redraw		(GtkCList	*clist);
 
 /* TreeView functions */
 
@@ -163,12 +172,17 @@ void gtkut_tree_view_scroll_to_cell	(GtkTreeView	*treeview,
 					 GtkTreePath	*path,
 					 gboolean	 align_center);
 
+void gtkut_tree_view_fast_clear		(GtkTreeView	*treeview,
+					 GtkTreeStore	*store);
+
 void gtkut_combo_set_items		(GtkCombo	*combo,
 					 const gchar	*str1, ...);
 
 gchar *gtkut_editable_get_selection	(GtkEditable	*editable);
 void gtkut_editable_disable_im		(GtkEditable	*editable);
 
+void gtkut_entry_strip_text		(GtkEntry	*entry);
+
 void gtkut_container_remove		(GtkContainer	*container,
 					 GtkWidget	*widget);
 
