@@ -17,10 +17,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/*
- * Edit VCard address book data.
- */
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -37,22 +33,24 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkhbbox.h>
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkstatusbar.h>
 #include <gtk/gtknotebook.h>
-#include <gtk/gtkfilesel.h>
 #include <gtk/gtkstock.h>
 
 #include "addrbook.h"
 #include "addressbook.h"
 #include "addressitem.h"
+#include "filesel.h"
 #include "gtkutils.h"
 #include "stock_pixmap.h"
 #include "prefs_common.h"
 #include "manage_window.h"
 #include "mgutils.h"
 #include "ldif.h"
+#include "codeconv.h"
 #include "utils.h"
 
 #define IMPORTLDIF_GUESS_NAME "LDIF Import"
@@ -99,7 +97,6 @@ static struct _ImpLdif_Dlg {
 	gboolean  cancelled;
 } impldif_dlg;
 
-static struct _AddressFileSelection _imp_ldif_file_selector_;
 static AddressBookFile *_importedBook_;
 static AddressIndex *_imp_addressIndex_;
 static LdifFile *_ldifFile_ = NULL;
@@ -329,8 +326,11 @@ static gboolean imp_ldif_file_move() {
 	}
 
 	if( ! errFlag ) {
+		gchar *sFSFile;
 		/* Read attribute list */
-		ldif_set_file( _ldifFile_, sFile );
+		sFSFile = conv_filename_from_utf8( sFile );
+		ldif_set_file( _ldifFile_, sFSFile );
+		g_free( sFSFile );
 		if( ldif_read_tags( _ldifFile_ ) == MGU_SUCCESS ) {
 			/* Load fields */
 			/* ldif_print_file( _ldifFile_, stdout ); */
@@ -367,6 +367,7 @@ static void imp_ldif_finish_show() {
 		sMsg = mgu_error2string( _ldifFile_->retVal );
 	}
 	imp_ldif_status_show( sMsg );
+	gtk_button_set_label(GTK_BUTTON(impldif_dlg.btnCancel), GTK_STOCK_CLOSE);
 	gtk_widget_grab_focus(impldif_dlg.btnCancel);
 }
 
@@ -419,55 +420,18 @@ static void imp_ldif_cancel( GtkWidget *widget, gpointer data ) {
 	gtk_main_quit();
 }
 
-static void imp_ldif_file_ok( GtkWidget *widget, gpointer data ) {
-	const gchar *sFile;
-	AddressFileSelection *afs;
-	GtkWidget *fileSel;
-
-	afs = ( AddressFileSelection * ) data;
-	fileSel = afs->fileSelector;
-	sFile = gtk_file_selection_get_filename( GTK_FILE_SELECTION(fileSel) );
-
-	afs->cancelled = FALSE;
-	gtk_entry_set_text( GTK_ENTRY(impldif_dlg.file_entry), sFile );
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( impldif_dlg.file_entry );
-}
-
-static void imp_ldif_file_cancel( GtkWidget *widget, gpointer data ) {
-	AddressFileSelection *afs = ( AddressFileSelection * ) data;
-	afs->cancelled = TRUE;
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( impldif_dlg.file_entry );
-}
-
-static void imp_ldif_file_select_create( AddressFileSelection *afs ) {
-	GtkWidget *fileSelector;
-
-	fileSelector = gtk_file_selection_new( _("Select LDIF File") );
-	gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(fileSelector) );
-	g_signal_connect( G_OBJECT (GTK_FILE_SELECTION(fileSelector)->ok_button),
-			  "clicked", G_CALLBACK (imp_ldif_file_ok), ( gpointer ) afs );
-	g_signal_connect( G_OBJECT (GTK_FILE_SELECTION(fileSelector)->cancel_button),
-			  "clicked", G_CALLBACK (imp_ldif_file_cancel), ( gpointer ) afs );
-	afs->fileSelector = fileSelector;
-	afs->cancelled = TRUE;
-}
-
 static void imp_ldif_file_select( void ) {
-	gchar *sFile;
-	if( ! _imp_ldif_file_selector_.fileSelector )
-		imp_ldif_file_select_create( & _imp_ldif_file_selector_ );
+	gchar *sSelFile;
 
-	sFile = gtk_editable_get_chars( GTK_EDITABLE(impldif_dlg.file_entry), 0, -1 );
-	gtk_file_selection_set_filename(
-		GTK_FILE_SELECTION( _imp_ldif_file_selector_.fileSelector ),
-		sFile );
-	g_free( sFile );
-	gtk_widget_show( _imp_ldif_file_selector_.fileSelector );
-	gtk_grab_add( _imp_ldif_file_selector_.fileSelector );
+	sSelFile = filesel_select_file( _("Select LDIF File"), NULL,
+					GTK_FILE_CHOOSER_ACTION_OPEN );
+	if ( sSelFile ) {
+		gchar *sUTF8File;
+		sUTF8File = conv_filename_to_utf8( sSelFile );
+		gtk_entry_set_text( GTK_ENTRY(impldif_dlg.file_entry), sUTF8File );
+		g_free( sUTF8File );
+		g_free( sSelFile );
+	}
 }
 
 static gint imp_ldif_delete_event( GtkWidget *widget, GdkEventAny *event, gpointer data ) {
@@ -597,6 +561,7 @@ static void imp_ldif_page_fields( gint pageNum, gchar *pageLbl ) {
 			GTK_CLIST(clist_field), FIELD_COL_FIELD, FIELDS_COL_WIDTH_FIELD );
 	gtk_clist_set_column_width(
 			GTK_CLIST(clist_field), FIELD_COL_ATTRIB, FIELDS_COL_WIDTH_ATTRIB );
+	gtkut_clist_set_redraw( GTK_CLIST(clist_field) );
 
 	for( i = 0; i < FIELDS_N_COLS; i++ )
 		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist_field)->column[i].button, GTK_CAN_FOCUS);
@@ -740,6 +705,7 @@ static void imp_ldif_dialog_create() {
 			 G_CALLBACK(imp_ldif_delete_event), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(imp_ldif_key_pressed), NULL);
+	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	vbox = gtk_vbox_new(FALSE, 4);
 	gtk_widget_show(vbox);
@@ -764,9 +730,12 @@ static void imp_ldif_dialog_create() {
 	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	/* Button panel */
-	gtkut_stock_button_set_create(&hbbox, &btnPrev, _("Prev"),
-				      &btnNext, _("Next"),
+	gtkut_stock_button_set_create(&hbbox, &btnNext, _("Next"),
+				      &btnPrev, _("Prev"),
 				      &btnCancel, GTK_STOCK_CANCEL);
+	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbbox), btnCancel,
+					   TRUE);
+	gtkut_box_set_reverse_order(GTK_BOX(hbbox), FALSE);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbbox), 2);
 	gtk_widget_grab_default(btnNext);
@@ -807,7 +776,6 @@ AddressBookFile *addressbook_imp_ldif( AddressIndex *addrIndex ) {
 	if( ! impldif_dlg.window )
 		imp_ldif_create();
 	impldif_dlg.cancelled = FALSE;
-	gtk_widget_show(impldif_dlg.window);
 	manage_window_set_transient(GTK_WINDOW(impldif_dlg.window));
 	gtk_widget_grab_default(impldif_dlg.btnNext);
 
@@ -819,6 +787,7 @@ AddressBookFile *addressbook_imp_ldif( AddressIndex *addrIndex ) {
 	gtk_notebook_set_current_page( GTK_NOTEBOOK(impldif_dlg.notebook), PAGE_FILE_INFO );
 	gtk_widget_set_sensitive( impldif_dlg.btnPrev, FALSE );
 	gtk_widget_set_sensitive( impldif_dlg.btnNext, TRUE );
+	gtk_button_set_label( GTK_BUTTON(impldif_dlg.btnCancel), GTK_STOCK_CANCEL );
 	stock_pixmap_gdk( impldif_dlg.window, STOCK_PIXMAP_MARK,
 			  &markxpm, &markxpmmask );
 	imp_ldif_message();
@@ -832,6 +801,9 @@ AddressBookFile *addressbook_imp_ldif( AddressIndex *addrIndex ) {
 	impldif_dlg.fileName = NULL;
 
 	_ldifFile_ = ldif_create();
+
+	gtk_widget_show(impldif_dlg.window);
+
 	gtk_main();
 	gtk_widget_hide(impldif_dlg.window);
 	ldif_free( _ldifFile_ );
