@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,7 +44,6 @@
 #include "account.h"
 #include "procmsg.h"
 #include "socket.h"
-#include "ssl.h"
 #include "pop.h"
 #include "recv.h"
 #include "mbox.h"
@@ -59,6 +58,7 @@
 #include "trayicon.h"
 #include "filter.h"
 #include "folder.h"
+#include "procheader.h"
 
 static GList *inc_dialog_list = NULL;
 
@@ -69,8 +69,6 @@ static GdkPixbuf *current_pixbuf;
 static GdkPixbuf *error_pixbuf;
 static GdkPixbuf *ok_pixbuf;
 
-#define MSGBUFSIZE	8192
-
 static void inc_finished		(MainWindow		*mainwin,
 					 gint			 new_messages);
 
@@ -546,7 +544,7 @@ static void inc_progress_dialog_set_list(IncProgressDialog *inc_dialog)
 		session->data = inc_dialog;
 		progress_dialog_append(inc_dialog->dialog, NULL,
 				       pop3_session->ac_prefs->account_name,
-				       _("Standby"), NULL);
+				       _("Standby"), "", NULL);
 	}
 }
 
@@ -616,6 +614,12 @@ static void inc_session_destroy(IncSession *session)
 	g_free(session);
 }
 
+static void inc_update_folder_foreach(GHashTable *table)
+{
+	procmsg_flush_folder_foreach(table);
+	folderview_update_item_foreach(table, !prefs_common.open_inbox_on_inc);
+}
+
 static gint inc_start(IncProgressDialog *inc_dialog)
 {
 	IncSession *session;
@@ -638,14 +642,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		}
 
 		pop3_session = POP3_SESSION(session->session); 
-		pop3_session->user = g_strdup(pop3_session->ac_prefs->userid);
-		if (pop3_session->ac_prefs->passwd)
-			pop3_session->pass =
-				g_strdup(pop3_session->ac_prefs->passwd);
-		else if (pop3_session->ac_prefs->tmp_pass)
-			pop3_session->pass =
-				g_strdup(pop3_session->ac_prefs->tmp_pass);
-		else {
+		if (!pop3_session->pass) {
 			gchar *pass;
 
 			if (inc_dialog->show_dialog)
@@ -672,12 +669,16 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		qlist = next;
 	}
 
-#define SET_PIXMAP_AND_TEXT(pixbuf, str)				\
+#define SET_PIXMAP_AND_TEXT(pixbuf, status, progress)			\
 {									\
 	progress_dialog_set_row_pixbuf(inc_dialog->dialog,		\
 				       inc_dialog->cur_row, pixbuf);	\
 	progress_dialog_set_row_status(inc_dialog->dialog,		\
-				       inc_dialog->cur_row, str);	\
+				       inc_dialog->cur_row, status);	\
+	if (progress)							\
+		progress_dialog_set_row_progress(inc_dialog->dialog,	\
+						 inc_dialog->cur_row,	\
+						 progress);		\
 }
 
 	for (; inc_dialog->queue_list != NULL; inc_dialog->cur_row++) {
@@ -686,7 +687,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 
 		if (session->inc_state == INC_CANCEL ||
 		    pop3_session->pass == NULL) {
-			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Cancelled"));
+			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Cancelled"), NULL);
 			inc_session_destroy(session);
 			inc_dialog->queue_list =
 				g_list_remove(inc_dialog->queue_list, session);
@@ -697,7 +698,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		progress_dialog_scroll_to_row(inc_dialog->dialog,
 					      inc_dialog->cur_row);
 
-		SET_PIXMAP_AND_TEXT(current_pixbuf, _("Retrieving"));
+		SET_PIXMAP_AND_TEXT(current_pixbuf, _("Retrieving"), NULL);
 
 		/* begin POP3 session */
 		inc_state = inc_pop3_session_do(session);
@@ -706,36 +707,37 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		case INC_SUCCESS:
 			if (pop3_session->cur_total_num > 0)
 				msg = g_strdup_printf
-					(_("Done (%d message(s) (%s) received)"),
+					(_("%d message(s) (%s) received"),
 					 pop3_session->cur_total_num,
 					 to_human_readable(pop3_session->cur_total_recv_bytes));
 			else
-				msg = g_strdup_printf(_("Done (no new messages)"));
-			SET_PIXMAP_AND_TEXT(ok_pixbuf, msg);
+				msg = g_strdup_printf(_("no new messages"));
+			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Done"), msg);
 			g_free(msg);
 			break;
 		case INC_CONNECT_ERROR:
 			SET_PIXMAP_AND_TEXT(error_pixbuf,
-					    _("Connection failed"));
+					    _("Connection failed"), NULL);
 			break;
 		case INC_AUTH_FAILED:
-			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Auth failed"));
+			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Auth failed"),
+					    NULL);
 			break;
 		case INC_LOCKED:
-			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Locked"));
+			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Locked"), NULL);
 			break;
 		case INC_ERROR:
 		case INC_NO_SPACE:
 		case INC_IO_ERROR:
 		case INC_SOCKET_ERROR:
 		case INC_EOF:
-			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Error"));
+			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Error"), NULL);
 			break;
 		case INC_TIMEOUT:
-			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Timeout"));
+			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Timeout"), NULL);
 			break;
 		case INC_CANCEL:
-			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Cancelled"));
+			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Cancelled"), NULL);
 			break;
 		default:
 			break;
@@ -744,10 +746,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		new_msgs += session->new_msgs;
 
 		if (!prefs_common.scan_all_after_inc) {
-			folder_item_scan_foreach(session->folder_table);
-			folderview_update_item_foreach
-				(session->folder_table,
-				 !prefs_common.open_inbox_on_inc);
+			inc_update_folder_foreach(session->folder_table);
 		}
 
 		if (pop3_session->error_val == PS_AUTHFAIL &&
@@ -826,8 +825,6 @@ static IncState inc_pop3_session_do(IncSession *session)
 {
 	Pop3Session *pop3_session = POP3_SESSION(session->session);
 	IncProgressDialog *inc_dialog = (IncProgressDialog *)session->data;
-	gchar *server;
-	gushort port;
 	gchar *buf;
 
 	debug_print(_("getting new messages of account %s...\n"),
@@ -842,21 +839,8 @@ static IncState inc_pop3_session_do(IncSession *session)
 	gtk_window_set_title(GTK_WINDOW(inc_dialog->dialog->window), buf);
 	g_free(buf);
 
-	server = pop3_session->ac_prefs->recv_server;
-#if USE_SSL
-	port = pop3_session->ac_prefs->set_popport ?
-		pop3_session->ac_prefs->popport :
-		pop3_session->ac_prefs->ssl_pop == SSL_TUNNEL ? 995 : 110;
-	SESSION(pop3_session)->ssl_type = pop3_session->ac_prefs->ssl_pop;
-	if (pop3_session->ac_prefs->ssl_pop != SSL_NONE)
-		SESSION(pop3_session)->nonblocking =
-			pop3_session->ac_prefs->use_nonblocking_ssl;
-#else
-	port = pop3_session->ac_prefs->set_popport ?
-		pop3_session->ac_prefs->popport : 110;
-#endif
-
-	buf = g_strdup_printf(_("Connecting to POP3 server: %s..."), server);
+	buf = g_strdup_printf(_("Connecting to POP3 server: %s..."),
+			      pop3_session->ac_prefs->recv_server);
 	log_message("%s\n", buf);
 	progress_dialog_set_label(inc_dialog->dialog, buf);
 	g_free(buf);
@@ -866,17 +850,22 @@ static IncState inc_pop3_session_do(IncSession *session)
 
 	GTK_EVENTS_FLUSH();
 
-	if (session_connect(SESSION(pop3_session), server, port) < 0) {
+	if (session_connect(SESSION(pop3_session),
+			    SESSION(pop3_session)->server,
+			    SESSION(pop3_session)->port) < 0) {
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
-			    server, port);
+			    SESSION(pop3_session)->server,
+			    SESSION(pop3_session)->port);
 		session->inc_state = INC_CONNECT_ERROR;
 		statusbar_pop_all();
 		return INC_CONNECT_ERROR;
 	}
 
 	while (session_is_connected(SESSION(pop3_session)) &&
-	       session->inc_state != INC_CANCEL)
+	       session->inc_state != INC_CANCEL) {
 		gtk_main_iteration();
+	}
+	log_window_flush();
 
 	if (session->inc_state == INC_SUCCESS) {
 		switch (pop3_session->error_val) {
@@ -973,7 +962,7 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 #if 0
 		if (session->msg[session->cur_msg].recv_time <
 			session->current_time) {
-			gchar buf[MSGBUFSIZE];
+			gchar buf[BUFFSIZE];
 			g_snprintf(buf, sizeof(buf), _("Deleting message %d"),
 				   session->cur_msg);
 			progress_dialog_set_label(dialog, buf);
@@ -991,7 +980,7 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 					     IncSession *inc_session)
 {
-	gchar buf[MSGBUFSIZE];
+	gchar buf[BUFFSIZE];
 	Pop3Session *pop3_session = POP3_SESSION(inc_session->session);
 	gchar *total_size_str;
 	gint64 cur_total;
@@ -1027,12 +1016,12 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 
 	if (pop3_session->cur_total_num > 0) {
 		g_snprintf(buf, sizeof(buf),
-			   _("Retrieving (%d message(s) (%s) received)"),
+			   _("%d message(s) (%s) received"),
 			   pop3_session->cur_total_num,
 			   to_human_readable
 			   (pop3_session->cur_total_recv_bytes));
-		progress_dialog_set_row_status(inc_dialog->dialog,
-					       inc_dialog->cur_row, buf);
+		progress_dialog_set_row_progress(inc_dialog->dialog,
+						 inc_dialog->cur_row, buf);
 	}
 }
 
@@ -1117,6 +1106,8 @@ static gint inc_recv_data_progressive(Session *session, guint cur_len,
 
 	if (!pop3_session->new_msg_exist) return 0;
 
+	gdk_threads_enter();
+
 	cur_total = pop3_session->cur_total_bytes + cur_len;
 	if (cur_total > pop3_session->total_bytes)
 		cur_total = pop3_session->total_bytes;
@@ -1126,6 +1117,7 @@ static gint inc_recv_data_progressive(Session *session, guint cur_len,
 	inc_progress_dialog_update_periodic(inc_dialog, inc_session);
 	inc_update_folderview_periodic(inc_dialog, inc_session);
 
+	gdk_threads_leave();
 	return 0;
 }
 
@@ -1140,11 +1132,14 @@ static gint inc_recv_data_finished(Session *session, guint len, gpointer data)
 
 	inc_recv_data_progressive(session, 0, 0, inc_session);
 
+	gdk_threads_enter();
+
 	if (POP3_SESSION(session)->state == POP3_LOGOUT) {
 		inc_progress_dialog_update(inc_dialog, inc_session);
 		inc_update_folderview(inc_dialog, inc_session);
 	}
 
+	gdk_threads_leave();
 	return 0;
 }
 
@@ -1165,14 +1160,18 @@ static gint inc_recv_message(Session *session, const gchar *msg, gpointer data)
 	case POP3_GETRANGE_LAST:
 	case POP3_GETRANGE_UIDL:
 	case POP3_GETSIZE_LIST:
+		gdk_threads_enter();
 		inc_progress_dialog_update(inc_dialog, inc_session);
+		gdk_threads_leave();
 		break;
 	case POP3_RETR:
 		inc_recv_data_progressive(session, 0, 0, inc_session);
 		break;
 	case POP3_LOGOUT:
+		gdk_threads_enter();
 		inc_progress_dialog_update(inc_dialog, inc_session);
 		inc_update_folderview(inc_dialog, inc_session);
+		gdk_threads_leave();
 		break;
 	default:
 		break;
@@ -1185,6 +1184,7 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 {
 	FolderItem *inbox;
 	GSList *cur;
+	MsgInfo *msginfo;
 	FilterInfo *fltinfo;
 	IncSession *inc_session = (IncSession *)(SESSION(session)->data);
 	gint val;
@@ -1192,6 +1192,8 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 
 	g_return_val_if_fail(inc_session != NULL, DROP_ERROR);
 
+	gdk_threads_enter();
+
 	if (session->ac_prefs->inbox) {
 		inbox = folder_find_item_from_identifier
 			(session->ac_prefs->inbox);
@@ -1199,55 +1201,75 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 			inbox = folder_get_default_inbox();
 	} else
 		inbox = folder_get_default_inbox();
-	if (!inbox)
+	if (!inbox) {
+		gdk_threads_leave();
 		return DROP_ERROR;
+	}
 
 	fltinfo = filter_info_new();
 	fltinfo->account = session->ac_prefs;
 	fltinfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
 	fltinfo->flags.tmp_flags = MSG_RECEIVED;
 
+	msginfo = procheader_parse_file(file, fltinfo->flags, FALSE);
+	if (!msginfo) {
+		g_warning("inc_drop_message: procheader_parse_file failed");
+		filter_info_free(fltinfo);
+		gdk_threads_leave();
+		return DROP_ERROR;
+	}
+	fltinfo->flags = msginfo->flags;
+	msginfo->file_path = g_strdup(file);
+
 	if (prefs_common.enable_junk &&
 	    prefs_common.filter_junk_on_recv &&
 	    prefs_common.filter_junk_before) {
-		filter_apply(prefs_common.junk_fltlist, file, fltinfo);
+		filter_apply_msginfo(prefs_common.junk_fltlist, msginfo,
+				     fltinfo);
 		if (fltinfo->drop_done)
 			is_junk = TRUE;
 		else if (fltinfo->error == FLT_ERROR_EXEC_FAILED) {
 			alertpanel_error
 				(_("Execution of the junk filter command failed.\n"
 				   "Please check the junk mail control setting."));
+			procmsg_msginfo_free(msginfo);
 			filter_info_free(fltinfo);
 			inc_session->inc_state = INC_ERROR;
+			gdk_threads_leave();
 			return DROP_ERROR;
 		}
 	}
 
 	if (!fltinfo->drop_done && session->ac_prefs->filter_on_recv)
-		filter_apply(prefs_common.fltlist, file, fltinfo);
+		filter_apply_msginfo(prefs_common.fltlist, msginfo, fltinfo);
 
 	if (!fltinfo->drop_done) {
 		if (prefs_common.enable_junk &&
 		    prefs_common.filter_junk_on_recv &&
 		    !prefs_common.filter_junk_before) {
-			filter_apply(prefs_common.junk_fltlist, file, fltinfo);
+			filter_apply_msginfo(prefs_common.junk_fltlist,
+					     msginfo, fltinfo);
 			if (fltinfo->drop_done)
 				is_junk = TRUE;
 			else if (fltinfo->error == FLT_ERROR_EXEC_FAILED) {
 				alertpanel_error
 					(_("Execution of the junk filter command failed.\n"
 					   "Please check the junk mail control setting."));
+				procmsg_msginfo_free(msginfo);
 				filter_info_free(fltinfo);
 				inc_session->inc_state = INC_ERROR;
+				gdk_threads_leave();
 				return DROP_ERROR;
 			}
 		}
 	}
 
 	if (!fltinfo->drop_done) {
-		if (folder_item_add_msg
-			(inbox, file, &fltinfo->flags, FALSE) < 0) {
+		msginfo->flags = fltinfo->flags;
+		if (folder_item_add_msg_msginfo(inbox, msginfo, FALSE) < 0) {
+			procmsg_msginfo_free(msginfo);
 			filter_info_free(fltinfo);
+			gdk_threads_leave();
 			return DROP_ERROR;
 		}
 		fltinfo->dest_list = g_slist_append(fltinfo->dest_list, inbox);
@@ -1275,8 +1297,10 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 			inc_session->new_msgs++;
 	}
 
+	procmsg_msginfo_free(msginfo);
 	filter_info_free(fltinfo);
 
+	gdk_threads_leave();
 	return val;
 }
 
@@ -1505,9 +1529,7 @@ static gint get_spool(FolderItem *dest, const gchar *mbox)
 	unlock_mbox(mbox, lockfd, LOCK_FLOCK);
 
 	if (!prefs_common.scan_all_after_inc) {
-		folder_item_scan_foreach(folder_table);
-		folderview_update_item_foreach
-			(folder_table, !prefs_common.open_inbox_on_inc);
+		inc_update_folder_foreach(folder_table);
 	}
 
 	g_hash_table_destroy(folder_table);
@@ -1564,13 +1586,18 @@ static gint inc_autocheck_func(gpointer data)
 {
 	MainWindow *mainwin = (MainWindow *)data;
 
+	gdk_threads_enter();
+
 	if (inc_lock_count) {
 		debug_print("autocheck is locked.\n");
 		inc_autocheck_timer_set_interval(1000);
+		gdk_threads_leave();
 		return FALSE;
 	}
 
 	inc_all_account_mail(mainwin, TRUE);
 
+	gdk_threads_leave();
+
 	return FALSE;
 }
