@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2010 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -85,6 +85,7 @@
 #include "plugin_manager.h"
 #include "foldersel.h"
 #include "update_check.h"
+#include "colorlabel.h"
 
 #if USE_GPGME
 #  include "rfc2015.h"
@@ -113,6 +114,10 @@ static gint lock_socket = -1;
 static gint lock_socket_tag = 0;
 static GIOChannel *lock_ch = NULL;
 
+#if USE_THREADS
+static GThread *main_thread;
+#endif
+
 static struct RemoteCmd {
 	gboolean receive;
 	gboolean receive_all;
@@ -132,6 +137,19 @@ static struct RemoteCmd {
 #endif
 } cmd;
 
+#define STATUSBAR_PUSH(mainwin, str) \
+{ \
+	gtk_statusbar_push(GTK_STATUSBAR(mainwin->statusbar), \
+			   mainwin->mainwin_cid, str); \
+	gtkut_widget_draw_now(mainwin->statusbar); \
+}
+
+#define STATUSBAR_POP(mainwin) \
+{ \
+	gtk_statusbar_pop(GTK_STATUSBAR(mainwin->statusbar), \
+			  mainwin->mainwin_cid); \
+}
+
 static void parse_cmd_opt		(int		 argc,
 					 char		*argv[]);
 
@@ -194,6 +212,7 @@ int main(int argc, char *argv[])
 	GList *iconlist = NULL;
 #endif
 	GObject *syl_app;
+	PrefsAccount *new_account = NULL;
 
 	app_init();
 	parse_cmd_opt(argc, argv);
@@ -208,6 +227,9 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
+#if USE_THREADS
+	gdk_threads_enter();
+#endif
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
 
@@ -217,13 +239,6 @@ int main(int argc, char *argv[])
 	gtk_widget_set_default_colormap(gdk_rgb_get_cmap());
 	gtk_widget_set_default_visual(gdk_rgb_get_visual());
 
-#if USE_THREADS || USE_LDAP
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-	if (!g_thread_supported())
-		g_error(_("g_thread is not supported by glib.\n"));
-#endif
-
 	parse_gtkrc_files();
 	setup_rc_dir();
 
@@ -243,9 +258,11 @@ int main(int argc, char *argv[])
 	CHDIR_EXIT_IF_FAIL(get_home_dir(), 1);
 
 	prefs_common_read_config();
+	filter_set_addressbook_func(addressbook_has_address);
 	filter_read_config();
 	prefs_actions_read_config();
 	prefs_display_header_read_config();
+	colorlabel_read_config();
 
 #ifdef G_OS_WIN32
 	{
@@ -314,17 +331,21 @@ int main(int argc, char *argv[])
 	main_window_reflect_prefs_all();
 
 	if (folder_read_list() < 0) {
-		setup(mainwin);
+		setup_mailbox();
 		folder_write_list();
 	}
 	if (!account_get_list()) {
-		account_edit_open();
-		account_add();
+		new_account = setup_account();
 	}
 
+	account_set_menu();
+	main_window_reflect_prefs_all();
+
 	account_set_missing_folder();
 	folder_set_missing_folders();
 	folderview_set(folderview);
+	if (new_account && new_account->folder)
+		folder_write_list();
 
 	addressbook_read_file();
 
@@ -337,10 +358,16 @@ int main(int argc, char *argv[])
 	g_signal_emit_by_name(syl_app, "init-done");
 
 	remote_command_exec();
+
+#if USE_UPDATE_CHECK
 	if (prefs_common.auto_update_check)
 		update_check(FALSE);
+#endif
 
 	gtk_main();
+#if USE_THREADS
+	gdk_threads_leave();
+#endif
 
 	return 0;
 }
@@ -580,10 +607,60 @@ static gint get_queued_message_num(void)
 	return queue->total;
 }
 
+#if USE_THREADS
+/* enables recursive locking with gdk_thread_enter / gdk_threads_leave */
+static GStaticRecMutex syl_mutex = G_STATIC_REC_MUTEX_INIT;
+
+static void thread_enter_func(void)
+{
+	g_static_rec_mutex_lock(&syl_mutex);
+#if 0
+	syl_mutex_lock_count++;
+	if (syl_mutex_lock_count > 1)
+		g_print("enter: syl_mutex_lock_count: %d\n", syl_mutex_lock_count);
+#endif
+}
+
+static void thread_leave_func(void)
+{
+#if 0
+	syl_mutex_lock_count--;
+	if (syl_mutex_lock_count > 0)
+		g_print("leave: syl_mutex_lock_count: %d\n", syl_mutex_lock_count);
+#endif
+	g_static_rec_mutex_unlock(&syl_mutex);
+}
+
+static void event_loop_iteration_func(void)
+{
+	if (g_thread_self() != main_thread) {
+		g_fprintf(stderr, "event_loop_iteration_func called from non-main thread (%p)\n", g_thread_self());
+		g_usleep(10000);
+		return;
+	}
+	gtk_main_iteration();
+}
+#endif
+
 static void app_init(void)
 {
+#if USE_THREADS
+	if (!g_thread_supported())
+		g_thread_init(NULL);
+	if (!g_thread_supported())
+		g_error("g_thread is not supported by glib.");
+	else {
+		gdk_threads_set_lock_functions(thread_enter_func,
+					       thread_leave_func);
+		gdk_threads_init();
+		main_thread = g_thread_self();
+	}
+#endif
 	syl_init();
 
+#if USE_THREADS
+	set_event_loop_func(event_loop_iteration_func);
+#endif
 	prog_version = PROG_VERSION;
 
 #ifdef G_OS_WIN32
@@ -751,8 +828,19 @@ static void idle_function_for_gpgme(void)
 static void check_gpg(void)
 {
 #if USE_GPGME
-	if (gpgme_check_version("0.4.5") &&
-	    !gpgme_engine_check_version(GPGME_PROTOCOL_OpenPGP)) {
+	const gchar *version;
+	gpgme_error_t err = 0;
+
+	version = gpgme_check_version("1.0.0");
+	if (version) {
+		debug_print("GPGME Version: %s\n", version);
+		err = gpgme_engine_check_version(GPGME_PROTOCOL_OpenPGP);
+		if (err)
+			debug_print("gpgme_engine_check_version: %s\n",
+				    gpgme_strerror(err));
+	}
+
+	if (version && !err) {
 		/* Also does some gpgme init */
 	        gpgme_engine_info_t engineInfo;
 
@@ -764,7 +852,7 @@ static void check_gpg(void)
 
 		if (!gpgme_get_engine_info(&engineInfo)) {
 			while (engineInfo) {
-				debug_print("GpgME Protocol: %s\n      Version: %s\n",
+				debug_print("GPGME Protocol: %s\n      Version: %s\n",
 					    gpgme_get_protocol_name
 						(engineInfo->protocol),
 					    engineInfo->version ?
@@ -1014,10 +1102,16 @@ static void plugin_init(void)
 
 	mainwin = main_window_get();
 
-	if (syl_plugin_init_lib() != 0)
+	STATUSBAR_PUSH(mainwin, _("Loading plug-ins..."));
+
+	if (syl_plugin_init_lib() != 0) {
+		STATUSBAR_POP(mainwin);
 		return;
+	}
 
 	syl_plugin_add_symbol("prog_version", prog_version);
+	syl_plugin_add_symbol("main_window_lock", main_window_lock);
+	syl_plugin_add_symbol("main_window_unlock", main_window_unlock);
 	syl_plugin_add_symbol("main_window_get", main_window_get);
 	syl_plugin_add_symbol("main_window_popup", main_window_popup);
 	syl_plugin_add_symbol("app_will_exit", app_will_exit);
@@ -1030,17 +1124,31 @@ static void plugin_init(void)
 			      folderview_get_selected_item);
 	syl_plugin_add_symbol("folderview_mail_popup_factory",
 			      mainwin->folderview->mail_factory);
+	syl_plugin_add_symbol("folderview_imap_popup_factory",
+			      mainwin->folderview->imap_factory);
+	syl_plugin_add_symbol("folderview_news_popup_factory",
+			      mainwin->folderview->news_factory);
 
 	syl_plugin_add_symbol("summaryview", mainwin->summaryview);
 	syl_plugin_add_symbol("summary_select_by_msgnum",
 			      summary_select_by_msgnum);
 	syl_plugin_add_symbol("summary_select_by_msginfo",
 			      summary_select_by_msginfo);
+	syl_plugin_add_symbol("summary_lock", summary_lock);
+	syl_plugin_add_symbol("summary_unlock", summary_unlock);
+	syl_plugin_add_symbol("summary_is_locked", summary_is_locked);
 
 	syl_plugin_add_symbol("messageview_create_with_new_window",
 			      messageview_create_with_new_window);
 	syl_plugin_add_symbol("messageview_show", messageview_show);
 
+	syl_plugin_add_symbol("compose_new", compose_new);
+	syl_plugin_add_symbol("compose_entry_set", compose_entry_set);
+	syl_plugin_add_symbol("compose_entry_append", compose_entry_append);
+	syl_plugin_add_symbol("compose_entry_get_text", compose_entry_get_text);
+	syl_plugin_add_symbol("compose_lock", compose_lock);
+	syl_plugin_add_symbol("compose_unlock", compose_unlock);
+
 	syl_plugin_add_symbol("foldersel_folder_sel",
 			      foldersel_folder_sel);
 	syl_plugin_add_symbol("foldersel_folder_sel_full",
@@ -1057,6 +1165,7 @@ static void plugin_init(void)
 			      manage_window_get_focus_window);
 
 	syl_plugin_add_symbol("inc_mail", inc_mail);
+	syl_plugin_add_symbol("inc_is_active", inc_is_active);
 	syl_plugin_add_symbol("inc_lock", inc_lock);
 	syl_plugin_add_symbol("inc_unlock", inc_unlock);
 
@@ -1073,6 +1182,7 @@ static void plugin_init(void)
 #else
 	syl_plugin_load_all(PLUGINDIR);
 #endif
+	STATUSBAR_POP(mainwin);
 }
 
 static gchar *get_socket_name(void)
@@ -1265,6 +1375,10 @@ static gboolean lock_socket_input_cb(GIOChannel *source, GIOCondition condition,
 	gint fd, sock;
 	gchar buf[BUFFSIZE];
 
+#if USE_THREADS
+	gdk_threads_enter();
+#endif
+
 	fd = g_io_channel_unix_get_fd(source);
 	sock = fd_accept(fd);
 	fd_gets(sock, buf, sizeof(buf));
@@ -1324,6 +1438,9 @@ static gboolean lock_socket_input_cb(GIOChannel *source, GIOCondition condition,
 		strretchomp(buf);
 		if (strlen(buf) < 6 || buf[4] != ' ') {
 			fd_close(sock);
+#if USE_THREADS
+			gdk_threads_leave();
+#endif
 			return TRUE;
 		}
 		open_message(buf + 5);
@@ -1334,6 +1451,10 @@ static gboolean lock_socket_input_cb(GIOChannel *source, GIOCondition condition,
 
 	fd_close(sock);
 
+#if USE_THREADS
+	gdk_threads_leave();
+#endif
+
 	return TRUE;
 }
 
