@@ -31,14 +31,20 @@ enum {
 	PLUGIN_LOAD,
 	PLUGIN_UNLOAD,
 	FOLDERVIEW_MENU_POPUP,
+	COMPOSE_CREATED,
+	COMPOSE_DESTROY,
 	LAST_SIGNAL
 };
 
 #define SAFE_CALL(func_ptr)		{ if (func_ptr) func_ptr(); }
 #define SAFE_CALL_RET(func_ptr)		(func_ptr ? func_ptr() : NULL)
+#define SAFE_CALL_RET_VAL(func_ptr, retval) \
+					(func_ptr ? func_ptr() : retval)
 #define SAFE_CALL_ARG1(func_ptr, arg1)	{ if (func_ptr) func_ptr(arg1); }
 #define SAFE_CALL_ARG1_RET(func_ptr, arg1) \
 				(func_ptr ? func_ptr(arg1) : NULL)
+#define SAFE_CALL_ARG1_RET_VAL(func_ptr, arg1, retval) \
+				(func_ptr ? func_ptr(arg1) : retval)
 #define SAFE_CALL_ARG2(func_ptr, arg1, arg2) \
 				{ if (func_ptr) func_ptr(arg1, arg2); }
 #define SAFE_CALL_ARG2_RET(func_ptr, arg1, arg2) \
@@ -99,6 +105,26 @@ static void syl_plugin_class_init(SylPluginClass *klass)
 			     G_TYPE_NONE,
 			     1,
 			     G_TYPE_POINTER);
+	plugin_signals[COMPOSE_CREATED] =
+		g_signal_new("compose-created",
+			     G_TYPE_FROM_CLASS(gobject_class),
+			     G_SIGNAL_RUN_FIRST,
+			     G_STRUCT_OFFSET(SylPluginClass, compose_created),
+			     NULL, NULL,
+			     g_cclosure_marshal_VOID__POINTER,
+			     G_TYPE_NONE,
+			     1,
+			     G_TYPE_POINTER);
+	plugin_signals[COMPOSE_DESTROY] =
+		g_signal_new("compose-destroy",
+			     G_TYPE_FROM_CLASS(gobject_class),
+			     G_SIGNAL_RUN_FIRST,
+			     G_STRUCT_OFFSET(SylPluginClass, compose_destroy),
+			     NULL, NULL,
+			     g_cclosure_marshal_VOID__POINTER,
+			     G_TYPE_NONE,
+			     1,
+			     G_TYPE_POINTER);
 }
 
 void syl_plugin_signal_connect(const gchar *name, GCallback callback,
@@ -117,7 +143,8 @@ void syl_plugin_signal_emit(const gchar *name, ...)
 	guint signal_id;
 
 	if (g_signal_parse_name(name, G_TYPE_FROM_INSTANCE(plugin_obj), &signal_id, NULL, FALSE)) {
-		va_list var_args;
+		 \
+				va_list var_args;
 		va_start(var_args, name);
 		g_signal_emit_valist(plugin_obj, signal_id, 0, var_args);
 		va_end(var_args);
@@ -172,12 +199,10 @@ gint syl_plugin_load(const gchar *name)
 
 	if (g_module_symbol(module, "plugin_load", (gpointer *)&load_func)) {
 		if (!syl_plugin_check_version(module)) {
-#if 0
 			g_warning("Version check failed. Skipping: %s", name);
 			g_module_close(module);
 			g_free(file);
 			return -1;
-#endif
 		}
 
 		debug_print("calling plugin_load() in %s\n",
@@ -284,6 +309,10 @@ gboolean syl_plugin_check_version(GModule *module)
 {
 	gint (*version_func)(void);
 	gint ver;
+	gint a_major;
+	gint a_minor;
+	gint p_major;
+	gint p_minor;
 
 	g_return_val_if_fail(module != NULL, FALSE);
 
@@ -298,7 +327,11 @@ gboolean syl_plugin_check_version(GModule *module)
 		return FALSE;
 	}
 
-	if (ver == SYL_PLUGIN_INTERFACE_VERSION) {
+	a_major = SYL_PLUGIN_INTERFACE_VERSION & 0xff00;
+	a_minor = SYL_PLUGIN_INTERFACE_VERSION & 0x00ff;
+	p_major = ver & 0xff00;
+	p_minor = ver & 0x00ff;
+	if (a_major == p_major && a_minor >= p_minor) {
 		debug_print("Version OK: plugin: %d, app: %d\n",
 			    ver, SYL_PLUGIN_INTERFACE_VERSION);
 		return TRUE;
@@ -327,6 +360,30 @@ const gchar *syl_plugin_get_prog_version(void)
 	return (gchar *)sym;
 }
 
+void syl_plugin_main_window_lock(void)
+{
+	void (*func)(gpointer);
+	gpointer mainwin;
+
+	mainwin = syl_plugin_main_window_get();
+	if (mainwin) {
+		func = syl_plugin_lookup_symbol("main_window_lock");
+		SAFE_CALL_ARG1(func, mainwin);
+	}
+}
+
+void syl_plugin_main_window_unlock(void)
+{
+	void (*func)(gpointer);
+	gpointer mainwin;
+
+	mainwin = syl_plugin_main_window_get();
+	if (mainwin) {
+		func = syl_plugin_lookup_symbol("main_window_unlock");
+		SAFE_CALL_ARG1(func, mainwin);
+	}
+}
+
 gpointer syl_plugin_main_window_get(void)
 {
 	gpointer (*func)(void);
@@ -370,6 +427,10 @@ static GtkItemFactory *get_item_factory(const gchar *path)
 		ifactory = syl_plugin_lookup_symbol("main_window_menu_factory");
 	else if (strncmp(path, "<MailFolder>", 12) == 0)
 		ifactory = syl_plugin_lookup_symbol("folderview_mail_popup_factory");
+	else if (strncmp(path, "<IMAPFolder>", 12) == 0)
+		ifactory = syl_plugin_lookup_symbol("folderview_imap_popup_factory");
+	else if (strncmp(path, "<NewsFolder>", 12) == 0)
+		ifactory = syl_plugin_lookup_symbol("folderview_news_popup_factory");
 	else
 		ifactory = syl_plugin_lookup_symbol("main_window_menu_factory");
 
@@ -551,6 +612,44 @@ void syl_plugin_open_message(const gchar *folder_id, guint msgnum)
 	}
 }
 
+void syl_plugin_summary_lock(void)
+{
+	void (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		func = syl_plugin_lookup_symbol("summary_lock");
+		SAFE_CALL_ARG1(func, summary);
+	}
+}
+
+void syl_plugin_summary_unlock(void)
+{
+	void (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		func = syl_plugin_lookup_symbol("summary_unlock");
+		SAFE_CALL_ARG1(func, summary);
+	}
+}
+
+gboolean syl_plugin_summary_is_locked(void)
+{
+	gboolean (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		func = syl_plugin_lookup_symbol("summary_is_locked");
+		return SAFE_CALL_ARG1_RET_VAL(func, summary, FALSE);
+	}
+
+	return FALSE;
+}
+
 gpointer syl_plugin_messageview_create_with_new_window(void)
 {
 	gpointer (*func)(void);
@@ -571,6 +670,60 @@ void syl_plugin_open_message_by_new_window(MsgInfo *msginfo)
 	}
 }
 
+
+gpointer syl_plugin_compose_new(PrefsAccount *account, FolderItem *item,
+				const gchar *mailto, GPtrArray *attach_files)
+{
+	gpointer (*func)(PrefsAccount *, FolderItem *, const gchar *,
+			 GPtrArray *);
+
+	func = syl_plugin_lookup_symbol("compose_new");
+	return SAFE_CALL_ARG4_RET(func, account, item, mailto, attach_files);
+}
+
+void syl_plugin_compose_entry_set(gpointer compose, const gchar *text,
+				  gint type)
+{
+	void (*func)(gpointer, const gchar *, gint);
+
+	func = syl_plugin_lookup_symbol("compose_entry_set");
+	SAFE_CALL_ARG3(func, compose, text, type);
+}
+
+void syl_plugin_compose_entry_append(gpointer compose, const gchar *text,
+				     gint type)
+{
+	void (*func)(gpointer, const gchar *, gint);
+
+	func = syl_plugin_lookup_symbol("compose_entry_append");
+	SAFE_CALL_ARG3(func, compose, text, type);
+}
+
+gchar *syl_plugin_compose_entry_get_text(gpointer compose, gint type)
+{
+	gchar * (*func)(gpointer, gint);
+
+	func = syl_plugin_lookup_symbol("compose_entry_get_text");
+	return SAFE_CALL_ARG2_RET(func, compose, type);
+}
+
+void syl_plugin_compose_lock(gpointer compose)
+{
+	void (*func)(gpointer);
+
+	func = syl_plugin_lookup_symbol("compose_lock");
+	SAFE_CALL_ARG1(func, compose);
+}
+
+void syl_plugin_compose_unlock(gpointer compose)
+{
+	void (*func)(gpointer);
+
+	func = syl_plugin_lookup_symbol("compose_unlock");
+	SAFE_CALL_ARG1(func, compose);
+}
+
+
 FolderItem *syl_plugin_folder_sel(Folder *cur_folder, gint sel_type,
 				  const gchar *default_folder)
 {
@@ -642,6 +795,14 @@ void syl_plugin_inc_mail(void)
 	SAFE_CALL_ARG1(func, syl_plugin_main_window_get());
 }
 
+gboolean syl_plugin_inc_is_active(void)
+{
+	gboolean (*func)(void);
+
+	func = syl_plugin_lookup_symbol("inc_is_active");
+	return SAFE_CALL_RET_VAL(func, FALSE);
+}
+
 void syl_plugin_inc_lock(void)
 {
 	void (*func)(void);
