@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,18 +27,13 @@
 #include "gtkutils.h"
 
 #include "icons/address.xpm"
-#include "icons/book.xpm"
 #include "icons/category.xpm"
 #include "icons/clip.xpm"
 #include "icons/complete.xpm"
 #include "icons/continue.xpm"
 #include "icons/deleted.xpm"
-#include "icons/dir-close.xpm"
-#include "icons/dir-open.xpm"
-#include "icons/dir-noselect.xpm"
 #include "icons/error.xpm"
 #include "icons/forwarded.xpm"
-#include "icons/group.xpm"
 #include "icons/interface.xpm"
 #include "icons/jpilot.xpm"
 #include "icons/ldap.xpm"
@@ -77,6 +72,13 @@
 #include "icons/stock_sylpheed_newmail.h"
 #include "icons/stock_sylpheed_newmail_16.h"
 #include "icons/sylpheed-logo.h"
+#include "icons/stock_person.h"
+#include "icons/stock_book.h"
+#include "icons/folder-close.h"
+#include "icons/folder-open.h"
+#include "icons/folder-noselect.h"
+#include "icons/folder-search.h"
+#include "icons/group.h"
 
 typedef struct _StockPixmapData	StockPixmapData;
 
@@ -95,18 +97,18 @@ struct _StockPixmapData
 static StockPixmapData pixmaps[] =
 {
 	{address_xpm	 , NULL, NULL},
-	{book_xpm	 , NULL, NULL},
+	{NULL, NULL, NULL, NULL, stock_book, sizeof(stock_book), "stock_book", 16},
 	{category_xpm	 , NULL, NULL},
 	{clip_xpm	 , NULL, NULL},
 	{complete_xpm	 , NULL, NULL},
 	{continue_xpm	 , NULL, NULL},
 	{deleted_xpm	 , NULL, NULL},
-	{dir_close_xpm	 , NULL, NULL},
-	{dir_open_xpm	 , NULL, NULL},
-	{dir_noselect_xpm, NULL, NULL},
+	{NULL, NULL, NULL, NULL, folder_close, sizeof(folder_close), "folder-close", 0},
+	{NULL, NULL, NULL, NULL, folder_open, sizeof(folder_open), "folder-open", 0},
+	{NULL, NULL, NULL, NULL, folder_noselect, sizeof(folder_noselect), "folder-noselect", 0},
 	{error_xpm	 , NULL, NULL},
 	{forwarded_xpm	 , NULL, NULL},
-	{group_xpm	 , NULL, NULL},
+	{NULL, NULL, NULL, NULL, group, sizeof(group), "group", 0},
 	{interface_xpm	 , NULL, NULL},
 	{jpilot_xpm	 , NULL, NULL},
 	{ldap_xpm	 , NULL, NULL},
@@ -146,6 +148,8 @@ static StockPixmapData pixmaps[] =
 	{NULL, NULL, NULL, NULL, stock_sylpheed_newmail, sizeof(stock_sylpheed_newmail), NULL, 0},
 	{NULL, NULL, NULL, NULL, stock_sylpheed_newmail_16, sizeof(stock_sylpheed_newmail_16), NULL, 0},
 	{NULL, NULL, NULL, NULL, sylpheed_logo, sizeof(sylpheed_logo), NULL, 0},
+	{NULL, NULL, NULL, NULL, stock_person, sizeof(stock_person), "stock_person", 16},
+	{NULL, NULL, NULL, NULL, folder_search, sizeof(folder_search), "folder-search", 0},
 };
 
 
