@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 #include <glib.h>
 #include <gtk/gtkwidget.h>
+#include <gtk/gtktexttag.h>
 
 typedef struct _TextView	TextView;
 
@@ -36,58 +37,52 @@ struct _TextView
 {
 	GtkWidget *vbox;
 	GtkWidget *scrolledwin;
-	GtkWidget *scrolledwin_sb;
-	GtkWidget *scrolledwin_mb;
 	GtkWidget *text;
-	GtkWidget *text_sb;
-	GtkWidget *text_mb;
 
 	GtkWidget *popup_menu;
 	GtkItemFactory *popup_factory;
 
-	gboolean text_is_mb;
+	GtkTextTag *quote0_tag;
+	GtkTextTag *quote1_tag;
+	GtkTextTag *quote2_tag;
+	GtkTextTag *link_tag;
+	GtkTextTag *hover_link_tag;
 
 	GSList *uri_list;
 	gint body_pos;
-	gint cur_pos;
-
-	GdkFont *msgfont;
-	GdkFont *boldfont;
 
 	gboolean show_all_headers;
 
 	MessageView *messageview;
 };
 
-TextView *textview_create	(void);
-void textview_init		(TextView	*textview);
+TextView *textview_create		(void);
+void textview_init			(TextView	*textview);
+void textview_reflect_prefs		(TextView	*textview);
+
 void textview_show_message	(TextView	*textview,
 				 MimeInfo	*mimeinfo,
 				 const gchar	*file);
 void textview_show_part		(TextView	*textview,
 				 MimeInfo	*mimeinfo,
 				 FILE		*fp);
-void textview_show_mime_part	(TextView	*textview,
-				 MimeInfo	*partinfo);
-#if USE_GPGME
-void textview_show_signature_part(TextView	*textview,
-				  MimeInfo 	*partinfo);
-#endif
+void textview_show_error	(TextView	*textview);
+
 void textview_clear		(TextView	*textview);
 void textview_destroy		(TextView	*textview);
+
 void textview_set_all_headers	(TextView	*textview,
 				 gboolean	 all_headers);
 void textview_set_font		(TextView	*textview,
 				 const gchar	*codeset);
 void textview_set_position	(TextView	*textview,
 				 gint		 pos);
+
 void textview_scroll_one_line	(TextView	*textview,
 				 gboolean	 up);
 gboolean textview_scroll_page	(TextView	*textview,
 				 gboolean	 up);
 
-void textview_update_message_colors	(void);
-
 gboolean textview_search_string			(TextView	*textview,
 						 const gchar	*str,
 						 gboolean	 case_sens);
